@@ -11,7 +11,7 @@
  *-------------------------------------------------------------------------
  */
 
-#define PG_CHECKSUMS_VERSION "0.11"
+#define PG_CHECKSUMS_VERSION "0.12"
 
 #include "postgres_fe.h"
 
@@ -40,9 +40,10 @@ static ControlFileData *ControlFile;
 static XLogRecPtr checkpointLSN;
 
 static char *only_filenode = NULL;
+static bool do_sync = true;
 static bool debug = false;
 static bool verbose = false;
-static bool show_progress = false;
+static bool showprogress = false;
 static bool online = false;
 
 char *DataDir = NULL;
@@ -73,15 +74,14 @@ static const char *progname;
  */
 int64		total_size = 0;
 int64		current_size = 0;
-instr_time	last_progress_update;
+instr_time	last_progress_report;
 instr_time	last_throttle;
 instr_time	scan_started;
 
 static void
 usage(void)
 {
-	printf(_("%s enables, disables or verifies data checksums in a PostgreSQL\n"), progname);
-	printf(_("database cluster.\n\n"));
+	printf(_("%s enables, disables, or verifies data checksums in a PostgreSQL database cluster.\n\n"), progname);
 	printf(_("Usage:\n"));
 	printf(_("  %s [OPTION]... [DATADIR]\n"), progname);
 	printf(_("\nOptions:\n"));
@@ -89,7 +89,8 @@ usage(void)
 	printf(_("  -c, --check              check data checksums (default)\n"));
 	printf(_("  -d, --disable            disable data checksums\n"));
 	printf(_("  -e, --enable             enable data checksums\n"));
-	printf(_("  -f, --filenode=FILENODE  check only relation with specified relfilenode\n"));
+	printf(_("  -f, --filenode=FILENODE  check only relation with specified filenode\n"));
+	printf(_("  -N, --no-sync            do not wait for changes to be written safely to disk\n"));
 	printf(_("  -P, --progress           show progress information\n"));
 	printf(_("      --max-rate=RATE      maximum I/O rate to verify or enable checksums\n"));
 	printf(_("                           (in MB/s)\n"));
@@ -97,8 +98,8 @@ usage(void)
 	printf(_("  -v, --verbose            output verbose messages\n"));
 	printf(_("  -V, --version            output version information, then exit\n"));
 	printf(_("  -?, --help               show this help, then exit\n"));
-	printf(_("\nIf no data directory (DATADIR) is specified, the environment "
-			 "variable PGDATA is used.\n\n"));
+	printf(_("\nIf no data directory (DATADIR) is specified, "
+			 "the environment variable PGDATA\nis used.\n\n"));
 	printf(_("Report bugs to https://github.com/credativ/pg_checksums/issues/new.\n"));
 }
 
@@ -131,7 +132,7 @@ update_checkpoint_lsn(void)
 #endif
 	if (!crc_ok)
 	{
-		fprintf(stderr, _("%s: pg_control CRC value is incorrect\n"), progname);
+		pg_log_error("pg_control CRC value is incorrect");
 		exit(1);
 	}
 #elif PG_VERSION_NUM >= 90600
@@ -148,28 +149,29 @@ static void
 toggle_progress_report(int signum)
 {
 
-	/* we handle SIGUSR1 only, and toggle the value of show_progress */
+	/* we handle SIGUSR1 only, and toggle the value of showprogress */
 	if (signum == SIGUSR1)
-		show_progress = !show_progress;
+		showprogress = !showprogress;
 
 }
 
 /*
  * Report current progress status and/or throttle. Parts borrowed from
- * PostgreSQLs' src/bin/pg_basebackup.c
+ * PostgreSQL's src/bin/pg_basebackup.c.
  */
 static void
-report_progress_or_throttle(bool force)
+progress_report_or_throttle(bool force)
 {
-	instr_time	now;
 	double		elapsed;
 	double		wait;
-	double		total_percent;
+	double		percent;
 	double		current_rate;
 	bool		skip_progress = false;
+	char		total_size_str[32];
+	char		current_size_str[32];
+	instr_time	now;
 
-	char		totalstr[32];
-	char		currentstr[32];
+	Assert(showprogress);
 
 	INSTR_TIME_SET_CURRENT(now);
 
@@ -180,7 +182,7 @@ report_progress_or_throttle(bool force)
 
 	/* Make sure we report at most once every 250 milliseconds */
 	if ((INSTR_TIME_GET_MILLISEC(now) -
-		 INSTR_TIME_GET_MILLISEC(last_progress_update) < 250) && !force)
+		 INSTR_TIME_GET_MILLISEC(last_progress_report) < 250) && !force)
 		skip_progress = true;
 
 	/* Save current time */
@@ -194,8 +196,8 @@ report_progress_or_throttle(bool force)
 	if (current_size > total_size)
 		total_size = current_size;
 
-	/* Calculate current percent done */
-	total_percent = total_size ? 100.0 * current_size / total_size : 0.0;
+	/* Calculate current percentage of size done */
+	percent = total_size ? 100.0 * current_size / total_size : 0.0;
 
 #define MEGABYTES (1024 * 1024)
 
@@ -205,9 +207,9 @@ report_progress_or_throttle(bool force)
 	 */
 	current_rate = (current_size / MEGABYTES) / (elapsed / 1000);
 
-	snprintf(totalstr, sizeof(totalstr), INT64_FORMAT,
+	snprintf(total_size_str, sizeof(total_size_str), INT64_FORMAT,
 			 total_size / MEGABYTES);
-	snprintf(currentstr, sizeof(currentstr), INT64_FORMAT,
+	snprintf(current_size_str, sizeof(current_size_str), INT64_FORMAT,
 			 current_size / MEGABYTES);
 
 	/* Throttle if desired */
@@ -219,8 +221,7 @@ report_progress_or_throttle(bool force)
 		 */
 		wait = (current_size / MEGABYTES / (maxrate / 1000)) - elapsed;
 		if (debug)
-			fprintf(stderr, _("%s: waiting for %f ms due to throttling\n"),
-					progname, wait);
+			pg_log_debug("waiting for %f ms due to throttling", wait);
 		pg_usleep(wait * 1000);
 		/* Recalculate current rate */
 		INSTR_TIME_SET_CURRENT(now);
@@ -229,18 +230,18 @@ report_progress_or_throttle(bool force)
 	}
 
 	/* Report progress if desired */
-	if (show_progress && !skip_progress)
+	if (showprogress && !skip_progress)
 	{
 		/*
 		 * Print five blanks at the end so the end of previous lines which were
 		 * longer don't remain partly visible.
 		 */
 		fprintf(stderr, "%s/%s MB (%d%%, %.0f MB/s)%5s",
-				currentstr, totalstr, (int)total_percent, current_rate, "");
+				current_size_str, total_size_str, (int)percent, current_rate, "");
 
 		/* Stay on the same line if reporting to a terminal */
 		fprintf(stderr, isatty(fileno(stderr)) ? "\r" : "\n");
-		last_progress_update = now;
+		last_progress_report = now;
 	}
 }
 
@@ -261,6 +262,7 @@ scan_file(const char *fn, BlockNumber segmentno)
 {
 	PGAlignedBlock buf;
 	PageHeader	header = (PageHeader) buf.data;
+	int			i;
 	int			f;
 	BlockNumber blockno;
 	int			flags;
@@ -282,8 +284,7 @@ scan_file(const char *fn, BlockNumber segmentno)
 			return;
 		}
 
-		fprintf(stderr, _("%s: could not open file \"%s\": %s\n"),
-				progname, fn, strerror(errno));
+		pg_log_error("could not open file \"%s\": %m", fn);
 		exit(1);
 	}
 
@@ -295,16 +296,14 @@ scan_file(const char *fn, BlockNumber segmentno)
 		int			r = read(f, buf.data, BLCKSZ);
 
 		if (debug && block_retry)
-			fprintf(stderr, _("%s: retrying block %u in file \"%s\"\n"),
-					progname, blockno, fn);
+			pg_log_debug("retrying block %u in file \"%s\"", blockno, fn);
 
 		if (r == 0)
 			break;
 		if (r < 0)
 		{
 			skippedfiles++;
-			fprintf(stderr, _("%s: could not read block %u in file \"%s\": %s\n"),
-					progname, blockno, fn, strerror(errno));
+			pg_log_error("could not read block %u in file \"%s\": %m", blockno, fn);
 			return;
 		}
 		if (r != BLCKSZ)
@@ -316,13 +315,12 @@ scan_file(const char *fn, BlockNumber segmentno)
 					/* We already tried once to reread the block, skip to the next block */
 					skippedblocks++;
 					if (debug)
-						fprintf(stderr, _("%s: retrying block %u in file \"%s\" failed, skipping to next block\n"),
-								progname, blockno, fn);
+						pg_log_debug("retrying block %u in file \"%s\" failed, skipping to next block",
+									 blockno, fn);
 
 					if (lseek(f, BLCKSZ-r, SEEK_CUR) == -1)
 					{
-						fprintf(stderr, _("%s: could not lseek to next block in file \"%s\": %m\n"),
-								progname, fn);
+						pg_log_error("could not lseek to next block in file \"%s\": %m", fn);
 						return;
 					}
 					continue;
@@ -340,8 +338,7 @@ scan_file(const char *fn, BlockNumber segmentno)
 				if (lseek(f, -r, SEEK_CUR) == -1)
 				{
 					skippedfiles++;
-					fprintf(stderr, _("%s: could not lseek in file \"%s\": %m\n"),
-							progname, fn);
+					pg_log_error("could not lseek to in file \"%s\": %m", fn);
 					return;
 				}
 
@@ -357,8 +354,8 @@ scan_file(const char *fn, BlockNumber segmentno)
 			{
 				/* Directly skip file if offline */
 				skippedfiles++;
-				fprintf(stderr, _("%s: could not read block %u in file \"%s\": read %d of %d\n"),
-						progname, blockno, fn, r, BLCKSZ);
+				pg_log_error("could not read block %u in file \"%s\": read %d of %d",
+							 blockno, fn, r, BLCKSZ);
 				return;
 			}
 		}
@@ -370,7 +367,7 @@ scan_file(const char *fn, BlockNumber segmentno)
 			/* Check for an all-zeroes page */
 			all_zeroes = true;
 			pagebytes = (size_t *) buf.data;
-			for (int i = 0; i < (BLCKSZ / sizeof(size_t)); i++)
+			for (i = 0; i < (BLCKSZ / sizeof(size_t)); i++)
 			{
 				if (pagebytes[i] != 0)
 				{
@@ -380,15 +377,14 @@ scan_file(const char *fn, BlockNumber segmentno)
 			}
 			if (!all_zeroes)
 			{
-				fprintf(stderr, _("%s: checksum verification failed in file \"%s\", block %u: pd_upper is zero but block is not all-zero\n"),
-						progname, fn, blockno);
+				pg_log_error("checksum verification failed in file \"%s\", block %u: pd_upper is zero but block is not all-zero",
+							 fn, blockno);
 				badblocks++;
 			}
 			else
 			{
 				if (debug && block_retry)
-					fprintf(stderr, _("%s: block %u in file \"%s\" is new, ignoring\n"),
-							progname, blockno, fn);
+					pg_log_debug("block %u in file \"%s\" is new, ignoring", blockno, fn);
 				skippedblocks++;
 			}
 			continue;
@@ -418,8 +414,7 @@ scan_file(const char *fn, BlockNumber segmentno)
 						if (lseek(f, -BLCKSZ, SEEK_CUR) == -1)
 						{
 							skippedfiles++;
-							fprintf(stderr, _("%s: could not lseek in file \"%s\": %m\n"),
-									progname, fn);
+							pg_log_error("could not lseek in file \"%s\": %m", fn);
 							return;
 						}
 
@@ -427,8 +422,8 @@ scan_file(const char *fn, BlockNumber segmentno)
 						block_retry = true;
 
 						if (debug)
-							fprintf(stderr, _("%s: checksum verification failed on first attempt in file \"%s\", block %u: calculated checksum %X but block contains %X\n"),
-									progname, fn, blockno, csum, header->pd_checksum);
+							pg_log_debug("checksum verification failed on first attempt in file \"%s\", block %u: calculated checksum %X but block contains %X",
+										 fn, blockno, csum, header->pd_checksum);
 
 						/* Reset loop to validate the block again */
 						blockno--;
@@ -457,8 +452,8 @@ scan_file(const char *fn, BlockNumber segmentno)
 						(PageGetLSN(buf.data) >> 32 == checkpointLSN >> 32))
 					{
 						if (debug)
-							fprintf(stderr, _("%s: block %u in file \"%s\" with LSN %X/%X is newer than checkpoint LSN %X/%X, ignoring\n"),
-									progname, blockno, fn, (uint32) (PageGetLSN(buf.data) >> 32), (uint32) PageGetLSN(buf.data), (uint32) (checkpointLSN >> 32), (uint32) checkpointLSN);
+							pg_log_debug("block %u in file \"%s\" with LSN %X/%X is newer than checkpoint LSN %X/%X, ignoring",
+										 blockno, fn, (uint32) (PageGetLSN(buf.data) >> 32), (uint32) PageGetLSN(buf.data), (uint32) (checkpointLSN >> 32), (uint32) checkpointLSN);
 						block_retry = false;
 						skippedblocks++;
 						continue;
@@ -466,13 +461,12 @@ scan_file(const char *fn, BlockNumber segmentno)
 				}
 
 				if (ControlFile->data_checksum_version == PG_DATA_CHECKSUM_VERSION)
-					fprintf(stderr, _("%s: checksum verification failed in file \"%s\", block %u: calculated checksum %X but block contains %X\n"),
-							progname, fn, blockno, csum, header->pd_checksum);
+					pg_log_error("checksum verification failed in file \"%s\", block %u: calculated checksum %X but block contains %X",
+								 fn, blockno, csum, header->pd_checksum);
 				badblocks++;
 			}
 			else if (block_retry && debug)
-				fprintf(stderr, _("%s: block %u in file \"%s\" verified ok on recheck\n"),
-						progname, blockno, fn);
+				pg_log_debug("block %u in file \"%s\" verified ok on recheck", blockno, fn);
 
 			block_retry = false;
 
@@ -485,35 +479,35 @@ scan_file(const char *fn, BlockNumber segmentno)
 			/* Seek back to beginning of block */
 			if (lseek(f, -BLCKSZ, SEEK_CUR) < 0)
 			{
-				fprintf(stderr, _("%s: seek failed for block %u in file \"%s\": %s\n"), progname, blockno, fn, strerror(errno));
+				pg_log_error("seek failed for block %u in file \"%s\": %m", blockno, fn);
 				exit(1);
 			}
 
 			/* Write block with checksum */
 			if (write(f, buf.data, BLCKSZ) != BLCKSZ)
 			{
-				fprintf(stderr, "%s: could not update checksum of block %u in file \"%s\": %s\n",
-						progname, blockno, fn, strerror(errno));
+				pg_log_error("could not write block %u in file \"%s\": %m",
+							 blockno, fn);
 				exit(1);
 			}
 		}
 
 		/* Report progress or throttle every 1024 blocks */
-		if ((show_progress || maxrate > 0) && (blockno % 1024 == 0))
-			report_progress_or_throttle(false);
+		if ((showprogress || maxrate > 0) && (blockno % 1024 == 0))
+			progress_report_or_throttle(false);
 	}
 
 	if (verbose)
 	{
 		if (mode == PG_MODE_CHECK)
-			fprintf(stderr, _("%s: checksums verified in file \"%s\"\n"), progname, fn);
+			pg_log_info("checksums verified in file \"%s\"", fn);
 		if (mode == PG_MODE_ENABLE)
-			fprintf(stderr, _("%s: checksums enabled in file \"%s\"\n"), progname, fn);
+			pg_log_info("checksums enabled in file \"%s\"", fn);
 	}
 
 	/* Make sure progress is reported at least once per file */
-	if (show_progress || maxrate > 0)
-		report_progress_or_throttle(false);
+	if (showprogress || maxrate > 0)
+		progress_report_or_throttle(false);
 
 	close(f);
 }
@@ -537,8 +531,7 @@ scan_directory(const char *basedir, const char *subdir, bool sizeonly)
 	dir = opendir(path);
 	if (!dir)
 	{
-		fprintf(stderr, _("%s: could not open directory \"%s\": %s\n"),
-				progname, path, strerror(errno));
+		pg_log_error("could not open directory \"%s\": %m", path);
 		exit(1);
 	}
 	while ((de = readdir(dir)) != NULL)
@@ -569,13 +562,11 @@ scan_directory(const char *basedir, const char *subdir, bool sizeonly)
 			{
 				/* File was removed in the meantime */
 				if (debug)
-					fprintf(stderr, _("%s: ignoring deleted file \"%s\"\n"),
-							progname, fn);
+					pg_log_debug("ignoring deleted file \"%s\"", fn);
 				continue;
 			}
 
-			fprintf(stderr, _("%s: could not stat file \"%s\": %s\n"),
-					progname, fn, strerror(errno));
+			pg_log_error("could not stat file \"%s\": %m", fn);
 			exit(1);
 		}
 		if (S_ISREG(st.st_mode))
@@ -641,9 +632,10 @@ main(int argc, char *argv[])
 		{"disable", no_argument, NULL, 'd'},
 		{"enable", no_argument, NULL, 'e'},
 		{"filenode", required_argument, NULL, 'f'},
-		{"verbose", no_argument, NULL, 'v'},
+		{"no-sync", no_argument, NULL, 'N'},
 		{"progress", no_argument, NULL, 'P'},
 		{"max-rate", required_argument, NULL, 1},
+		{"verbose", no_argument, NULL, 'v'},
 		{"debug", no_argument, NULL, 2},
 		{NULL, 0, NULL, 0}
 	};
@@ -654,9 +646,7 @@ main(int argc, char *argv[])
 	bool		crc_ok;
 #endif
 
-#if PG_VERSION_NUM >= 120000
 	pg_logging_init(argv[0]);
-#endif
 	set_pglocale_pgservice(argv[0], PG_TEXTDOMAIN("pg_checksums"));
 	progname = get_progname(argv[0]);
 
@@ -674,7 +664,7 @@ main(int argc, char *argv[])
 		}
 	}
 
-	while ((c = getopt_long(argc, argv, "abcD:def:Pv", long_options, &option_index)) != -1)
+	while ((c = getopt_long(argc, argv, "abcD:def:NPv", long_options, &option_index)) != -1)
 	{
 		switch (c)
 		{
@@ -696,11 +686,14 @@ main(int argc, char *argv[])
 			case 'f':
 				if (atoi(optarg) == 0)
 				{
-					fprintf(stderr, _("%s: invalid filenode specification, must be numeric: %s\n"), progname, optarg);
+					pg_log_error("invalid filenode specification, must be numeric: %s", optarg);
 					exit(1);
 				}
 				only_filenode = pstrdup(optarg);
 				break;
+			case 'N':
+				do_sync = false;
+				break;
 			case 'v':
 				verbose = true;
 				break;
@@ -708,12 +701,12 @@ main(int argc, char *argv[])
 				DataDir = optarg;
 				break;
 			case 'P':
-				show_progress = true;
+				showprogress = true;
 				break;
 			case 1:
 				if (atof(optarg) == 0)
 				{
-					fprintf(stderr, _("%s: invalid max-rate specification, must be numeric: %s\n"), progname, optarg);
+					pg_log_error("invalid max-rate specification, must be numeric: %s", optarg);
 					exit(1);
 				}
 				maxrate = atof(optarg);
@@ -738,7 +731,7 @@ main(int argc, char *argv[])
 		/* If no DataDir was specified, and none could be found, error out */
 		if (DataDir == NULL)
 		{
-			fprintf(stderr, _("%s: no data directory specified\n"), progname);
+			pg_log_error("no data directory specified");
 			fprintf(stderr, _("Try \"%s --help\" for more information.\n"), progname);
 			exit(1);
 		}
@@ -747,17 +740,17 @@ main(int argc, char *argv[])
 	/* Complain if any arguments remain */
 	if (optind < argc)
 	{
-		fprintf(stderr, _("%s: too many command-line arguments (first is \"%s\")\n"),
-				progname, argv[optind]);
+		pg_log_error("too many command-line arguments (first is \"%s\")",
+					 argv[optind]);
 		fprintf(stderr, _("Try \"%s --help\" for more information.\n"),
 				progname);
 		exit(1);
 	}
 
-	/* filenode checking only works in check mode */
+	/* filenode checking only works in --check mode */
 	if (mode != PG_MODE_CHECK && only_filenode)
 	{
-		fprintf(stderr, _("%s: --filenode option only possible with --check\n"), progname);
+		pg_log_error("option -f/--filenode can only be used with --check");
 		fprintf(stderr, _("Try \"%s --help\" for more information.\n"),
 				progname);
 		exit(1);
@@ -772,7 +765,7 @@ main(int argc, char *argv[])
 #endif
 	if (!crc_ok)
 	{
-		fprintf(stderr, _("%s: pg_control CRC value is incorrect\n"), progname);
+		pg_log_error("pg_control CRC value is incorrect");
 		exit(1);
 	}
 #elif PG_VERSION_NUM >= 90600
@@ -783,14 +776,13 @@ main(int argc, char *argv[])
 
 	if (ControlFile->pg_control_version != PG_CONTROL_VERSION)
 	{
-		fprintf(stderr, _("%s: cluster is not compatible with this version of pg_checksums\n"),
-				progname);
+		pg_log_error("cluster is not compatible with this version of pg_checksums");
 		exit(1);
 	}
 
 	if (ControlFile->blcksz != BLCKSZ)
 	{
-		fprintf(stderr, _("%s: database cluster is not compatible\n"), progname);
+		pg_log_error("database cluster is not compatible");
 		fprintf(stderr, _("The database cluster was initialized with block size %u, but pg_checksums was compiled with block size %u.\n"),
 				ControlFile->blcksz, BLCKSZ);
 		exit(1);
@@ -805,7 +797,7 @@ main(int argc, char *argv[])
 	{
 		if (mode != PG_MODE_CHECK)
 		{
-			fprintf(stderr, _("%s: cluster must be shut down\n"), progname);
+			pg_log_error("cluster must be shut down");
 			exit(1);
 		}
 		online = true;
@@ -814,29 +806,29 @@ main(int argc, char *argv[])
 	if (debug)
 	{
 		if (online)
-			fprintf(stderr, _("%s: online mode\n"), progname);
+			pg_log_debug("online mode");
 		else
-			fprintf(stderr, _("%s: offline mode\n"), progname);
+			pg_log_debug("offline mode");
 	}
 
 	if (ControlFile->data_checksum_version == 0 &&
 		mode == PG_MODE_CHECK)
 	{
-		fprintf(stderr, _("%s: data checksums are not enabled in cluster\n"), progname);
+		pg_log_error("data checksums are not enabled in cluster");
 		exit(1);
 	}
 
 	if (ControlFile->data_checksum_version == 0 &&
 		mode == PG_MODE_DISABLE)
 	{
-		fprintf(stderr, _("%s: data checksums are already disabled in cluster\n"), progname);
+		pg_log_error("data checksums are already disabled in cluster");
 		exit(1);
 	}
 
 	if (ControlFile->data_checksum_version > 0 &&
 		mode == PG_MODE_ENABLE)
 	{
-		fprintf(stderr, _("%s: data checksums are already enabled in cluster\n"), progname);
+		pg_log_error("data checksums are already enabled in cluster");
 		exit(1);
 	}
 
@@ -860,7 +852,7 @@ main(int argc, char *argv[])
 		 * legwork.
 		 */
 		if (debug)
-			fprintf(stderr, _("%s: acquiring data for progress reporting\n"), progname);
+			pg_log_debug("acquiring data for progress reporting");
 
 		total_size = scan_directory(DataDir, "global", true);
 		total_size += scan_directory(DataDir, "base", true);
@@ -868,10 +860,10 @@ main(int argc, char *argv[])
 
 		/*
 		 * Remember start time. Required to calculate the current rate in
-		 * report_progress_or_throttle().
+		 * progress_report_or_throttle().
 		 */
 		if (debug)
-			fprintf(stderr, _("%s: starting scan\n"), progname);
+			pg_log_debug("starting scan");
 		INSTR_TIME_SET_CURRENT(scan_started);
 
 		(void) scan_directory(DataDir, "global", false);
@@ -882,25 +874,26 @@ main(int argc, char *argv[])
 		 * Done. Move to next line in case progress information was shown.
 		 * Otherwise we clutter the summary output.
 		 */
-		if (show_progress)
+		if (showprogress)
 		{
-			report_progress_or_throttle(true);
+			progress_report_or_throttle(true);
 			if (isatty(fileno(stderr)))
 				fprintf(stderr, "\n");
 		}
 
 		printf(_("Checksum operation completed\n"));
-		printf(_("Files scanned:  %" INT64_MODIFIER "d\n"), files);
+		printf(_("Files scanned:  %s\n"), psprintf(INT64_FORMAT, files));
 		if (skippedfiles > 0)
-			printf(_("Files skipped: %" INT64_MODIFIER "d\n"), skippedfiles);
-		printf(_("Blocks scanned: %" INT64_MODIFIER "d\n"), blocks);
+			printf(_("Files skipped:  %s\n"), psprintf(INT64_FORMAT, skippedfiles));
+		printf(_("Blocks scanned: %s\n"), psprintf(INT64_FORMAT, blocks));
 		if (skippedblocks > 0)
-			printf(_("Blocks skipped: %" INT64_MODIFIER "d\n"), skippedblocks);
+			printf(_("Blocks skipped: %s\n"), psprintf(INT64_FORMAT, skippedblocks));;
 
 		if (mode == PG_MODE_CHECK)
 		{
-			printf(_("Bad checksums:  %" INT64_MODIFIER "d\n"), badblocks);
+			printf(_("Bad checksums:  %s\n"), psprintf(INT64_FORMAT, badblocks));
 			printf(_("Data checksum version: %d\n"), ControlFile->data_checksum_version);
+
 			if (badblocks > 0)
 				exit(1);
 
@@ -921,12 +914,17 @@ main(int argc, char *argv[])
 		ControlFile->data_checksum_version =
 			(mode == PG_MODE_ENABLE) ? PG_DATA_CHECKSUM_VERSION : 0;
 
-		updateControlFile(DataDir, ControlFile);
+		if (do_sync)
+		{
+			pg_log_info("syncing data directory");
 #if PG_VERSION_NUM >= 120000
-		fsync_pgdata(DataDir, PG_VERSION_NUM);
+			fsync_pgdata(DataDir, PG_VERSION_NUM);
 #else
-		fsync_pgdata(DataDir, progname, PG_VERSION_NUM);
+			fsync_pgdata(DataDir, progname, PG_VERSION_NUM);
 #endif
+		}
+		pg_log_info("updating control file");
+		updateControlFile(DataDir, ControlFile, do_sync);
 
 		if (verbose)
 			printf(_("Data checksum version: %d\n"), ControlFile->data_checksum_version);
